unit uAskUser;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, UITypes, Buttons;

type

   { TDialogIconButton }

   TDialogIconButton = class(TCollectionItem)
   private
     FCaption: string;
     FIcon: TMsgDlgBtn;
     FModalResult: LongInt;
     function GetCancel: Boolean;
     function GetDefault: Boolean;
     procedure SetCancel(const AValue: Boolean);
     procedure SetDefault(const AValue: Boolean);
     procedure SetIcon(AValue: TMsgDlgBtn);
   protected
     function GetDisplayName: string; override;
     procedure SetCaption(const AValue: string); virtual;
   public
     constructor Create(ACollection: TCollection); override;
     property Cancel: Boolean read GetCancel write SetCancel;
     property Caption: string read FCaption write SetCaption;
     property Default: Boolean read GetDefault write SetDefault;
     property Icon: TMsgDlgBtn read FIcon write SetIcon;
     property ModalResult: LongInt read FModalResult write FModalResult;
   end;

   { TDialogIconButtons }

   TDialogIconButtons = class(TCollection)
   protected
     FCancelButton: TDialogIconButton;
     FDefaultButton: TDialogIconButton;
     function GetItem(Index: Integer): TDialogIconButton;
     procedure SetCancelButton(const AValue: TDialogIconButton); virtual;
     procedure SetDefaultButton(const Value: TDialogIconButton); virtual;
     procedure SetItem(Index: Integer; const Value: TDialogIconButton);
   public
     function Add: TDialogIconButton;
     function FindButton(AModalResult: LongInt): TDialogIconButton;
     function FindButton(Order: array of LongInt): TDialogIconButton;
     property DefaultButton: TDialogIconButton read FDefaultButton write SetDefaultButton;
     property CancelButton: TDialogIconButton read FCancelButton write SetCancelButton;
     property Items[Index: Integer]: TDialogIconButton read GetItem write SetItem; default;
   end;

   { TDlgIconCaptionButton }

   TDlgIconCaptionButton = class(TCustomBitBtn)
   public
     constructor Create(TheOwner: TComponent; const aCaption: String; aKind: TBitBtnKind); reintroduce;
   end;


  { TAskDialog }

  TAskDialog = class(TForm)
  public
    constructor Create(const aCaption, aMsg: String; aDlgID: Integer; aDlgIconBtns: TDialogIconButtons); reintroduce;
  end;

  function AskUser(const aCaption, aMsg: String; aDlgID: Integer; aDlgIconBtns: TDialogIconButtons): TModalResult;

  function MyQuestionDlg(const aCaption, aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint): TModalResult;

implementation

uses Dialogs, Themes;

function AskUser(const aCaption, aMsg: String; aDlgID: Integer; aDlgIconBtns: TDialogIconButtons): TModalResult;
var
  dlg: TAskDialog;
  mr: Integer;
begin
  dlg := TAskDialog.Create(aCaption, aMsg, aDlgID, aDlgIconBtns);
  try
    mr := dlg.ShowModal;
  finally
    dlg.Free;
  end;
end;

function MyQuestionDlg(const aCaption, aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint): TModalResult;
begin
    with CreateMessageDialog(aMsg, DlgType, Buttons) do
      try
        Caption := ACaption;
        HelpContext := HelpCtx;
        Result := ShowModal;
      finally
        Free;
      end;
end;

{ TDlgIconCaptionButton }

constructor TDlgIconCaptionButton.Create(TheOwner: TComponent;
  const aCaption: String; aKind: TBitBtnKind);
begin
  inherited Create(TheOwner);
  //buttons.LoadGlyphFromStock(FButtonGlyph, );
end;

{ TAskDialog }

constructor TAskDialog.Create(const aCaption, aMsg: String; aDlgID: Integer;
  aDlgIconBtns: TDialogIconButtons);
begin
  inherited CreateNew(Nil);

end;

{ TDialogIconButtons }

function TDialogIconButtons.GetItem(Index: Integer): TDialogIconButton;
begin
  Result := TDialogIconButton(inherited GetItem(Index));
end;

procedure TDialogIconButtons.SetCancelButton(const AValue: TDialogIconButton);
begin
  FCancelButton := AValue;
end;

procedure TDialogIconButtons.SetDefaultButton(const Value: TDialogIconButton);
begin
  FDefaultButton := Value;
end;

procedure TDialogIconButtons.SetItem(Index: Integer; const Value: TDialogIconButton);
begin
  inherited SetItem(Index, Value);
end;

function TDialogIconButtons.Add: TDialogIconButton;
begin
  Exit(TDialogIconButton(inherited Add));
end;

function TDialogIconButtons.FindButton(AModalResult: LongInt): TDialogIconButton;
var
  i: Integer;
begin
  for i := 0 to Count - 1 do
    if Items[i].ModalResult = AModalResult then
      Exit(Items[i]);
  Result := Nil;
end;

function TDialogIconButtons.FindButton(Order: array of LongInt): TDialogIconButton;
var
  i: Integer;
begin
  for i := Low(Order) to High(Order) do
  begin
    Result := FindButton(Order[i]);
    if Result <> Nil then
      Exit;
  end;
  Result := Nil;
end;

{ TDialogIconButton }

function TDialogIconButton.GetCancel: Boolean;
begin
  Result := TDialogIconButtons(Collection).CancelButton = Self;
end;

function TDialogIconButton.GetDefault: Boolean;
begin
  Result := TDialogIconButtons(Collection).DefaultButton = Self;
end;

procedure TDialogIconButton.SetCancel(const AValue: Boolean);
begin
  if AValue then
    TDialogIconButtons(Collection).CancelButton := Self
  else
    TDialogIconButtons(Collection).CancelButton := Nil;
end;

procedure TDialogIconButton.SetDefault(const AValue: Boolean);
begin
  if AValue then
    TDialogIconButtons(Collection).DefaultButton := Self
  else
    TDialogIconButtons(Collection).DefaultButton := Nil;
end;

procedure TDialogIconButton.SetIcon(AValue: TMsgDlgBtn);
begin
  if FIcon <> AValue then
    FIcon := AValue;
end;

function TDialogIconButton.GetDisplayName: string;
begin
  Result := FCaption;
end;

procedure TDialogIconButton.SetCaption(const AValue: string);
begin
  FCaption := AValue;
end;

constructor TDialogIconButton.Create(ACollection: TCollection);
begin
  inherited Create(ACollection);
  FCaption := '';
  FModalResult := 0;
  FIcon := mbClose;
end;

end.

