unit uDlgButtons;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, UITypes, uAskUser;

function QuestionDlgBtns(const aCaption, aMsg: string; DlgType: TMsgDlgType; Buttons: array of const): TModalResult;

implementation

uses LazLogger, LazStringUtils, LCLType;

const
  DialogIds : Array[mtWarning..mtCustom] of Longint = (idDialogWarning,
    idDialogError, idDialogInfo, idDialogConfirm, idDialogBase);

function QuestionDlgBtns(const aCaption, aMsg: string; DlgType: TMsgDlgType; Buttons: array of const): TModalResult;

  function GetNextCaption(var i: Integer; out aCaption: String): Boolean;
  begin
    aCaption:='';
    if (i > High(Buttons)) then
      Exit(False);
    Result:=True;
    case Buttons[i].VType of
      vtString:     aCaption := Buttons[i].VString^;
      vtAnsiString: aCaption := AnsiString(Buttons[i].VAnsiString);
      vtChar:       aCaption := Buttons[i].VChar;
      vtPChar:      aCaption := Buttons[i].VPChar;
      vtPWideChar:  aCaption := Buttons[i].VPWideChar;
      vtWideChar:   aCaption := AnsiString(Buttons[i].VWideChar);
      vtWidestring: aCaption := AnsiString(WideString(Buttons[i].VWideString));
      else Result := False;
    end;
    if Result then
      Inc(i);
  end;

var
  btnCaption, s, lowerS: String;
  curBtnValue: TModalResult;
  iconKind: TMsgDlgBtn;
  dialogButtons: TDialogIconButtons;
  i: integer;
  isDefault, isCancel: Boolean;
  newButton: TDialogIconButton;

begin
  dialogButtons := TDialogIconButtons.Create(TDialogIconButton);
  try
    i := Low(Buttons);
    while i <= High(Buttons) do
      begin
        if Buttons[i].VType <> vtInteger then
          raise Exception.Create('TQuestionDlg.CreateQuestionDlg integer expected at ' + IntToStr(i) +
                                 ' but VType='+IntToStr(Ord(Buttons[i].VType))+' found.');
        if Buttons[i].VType = vtInteger then
          begin
            curBtnValue := Buttons[i].VInteger;    // get TModalResult
            Inc(i);

            BtnCaption := '';                     // get button caption and flags
            IsDefault := False;
            IsCancel := False;
            iconKind := mbClose;

            while GetNextCaption(i, s) do
              begin
                lowerS := LowerCase(s);
                case lowerS of
                  'isdefault': begin
                                 if DialogButtons.DefaultButton <> Nil then
                                   raise Exception.Create('TQuestionDlg.CreateQuestionDlg only one button can be default');
                                 IsDefault := True;
                               end;
                  'iscancel': begin
                                 if DialogButtons.DefaultButton <> Nil then
                                   raise Exception.Create('TQuestionDlg.CreateQuestionDlg only one button can be cancel');
                                 IsCancel := True;
                               end;
                  'mbYes':      iconKind := mbYes;
                  'mbNo':       iconKind := mbNo;
                  'mbOK':       iconKind := mbOK;
                  'mbCancel':   iconKind := mbCancel;
                  'mbAbort':    iconKind := mbAbort;
                  'mbRetry':    iconKind := mbRetry;
                  'mbIgnore':   iconKind := mbIgnore;
                  'mbAll':      iconKind := mbAll;
                  'mbNoToAll':  iconKind := mbNoToAll;
                  'mbYesToAll': iconKind := mbYesToAll;
                  'mbHelp':     iconKind := mbHelp;
                  'mbClose':    iconKind := mbClose;
                  else btnCaption := s;
                end;
                  //raise Exception.Create('TQuestionDlg.CreateQuestionDlg option expected at '+IntToStr(i)+', but found "'+s+'"');
              end;
          end;  // if ... vtInteger;
                //if BtnCaption = '' then
                  //raise Exception.Create('TQuestionDlg.Create: missing Button caption '+dbgs(i-1));

        NewButton := dialogButtons.Add;
        with NewButton do
          begin
            Caption := BtnCaption;
            ModalResult := CurBtnValue;
            Icon := iconKind;
          end;
          if IsDefault then
            DialogButtons.DefaultButton := NewButton;
          if IsCancel then
            DialogButtons.CancelButton := NewButton;
      end;
    if DialogButtons.DefaultButton = Nil then
      DialogButtons.DefaultButton := DialogButtons.FindButton([mrYes, mrOk, mrYesToAll, mrAll, mrRetry, mrCancel, mrNo, mrNoToAll, mrAbort, mrIgnore]);
    Result := AskUser(aCaption, ConvertLineEndings(aMsg), DialogIds[DlgType], DialogButtons);
  finally
    dialogButtons.Free;
  end;
end;

end.

